@@ -31,13 +31,14 @@
 import socket
 from typing import Optional
 
-from startrek.net.channel import is_opened, is_connected
+from startrek.types import Address
+from startrek.net.channel import is_opened, is_bound, is_connected
 from startrek import BaseChannel, ChannelReader, ChannelWriter
 
 
 class PackageChannelReader(ChannelReader):
 
-    def _receive_from(self, max_len: int) -> (Optional[bytes], Optional[tuple]):
+    def _receive_from(self, max_len: int) -> (Optional[bytes], Optional[Address]):
         sock = self.sock
         try:
             data, remote = sock.recvfrom(max_len)
@@ -58,7 +59,7 @@ class PackageChannelReader(ChannelReader):
         return data, remote
 
     # Override
-    def receive(self, max_len: int) -> (Optional[bytes], Optional[tuple]):
+    def receive(self, max_len: int) -> (Optional[bytes], Optional[Address]):
         remote = self.remote_address
         if remote is None:
             # not connect (UDP)
@@ -71,7 +72,7 @@ class PackageChannelReader(ChannelReader):
 
 class PackageChannelWriter(ChannelWriter):
 
-    def _sent_to(self, data: bytes, target: tuple) -> int:
+    def _sent_to(self, data: bytes, target: Address) -> int:
         sock = self.sock
         try:
             return sock.sendto(data, target)
@@ -85,7 +86,7 @@ class PackageChannelWriter(ChannelWriter):
                 raise error
 
     # Override
-    def send(self, data: bytes, target: tuple) -> int:
+    def send(self, data: bytes, target: Address) -> int:
         remote = self.remote_address
         if remote is None:
             # not connect (UDP)
@@ -100,27 +101,38 @@ class PackageChannelWriter(ChannelWriter):
 class PackageChannel(BaseChannel):
     """ Discrete Package Channel """
 
-    def __init__(self, remote: Optional[tuple], local: Optional[tuple], sock: socket.socket):
-        super().__init__(remote=remote, local=local, sock=sock)
+    def __init__(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket):
+        super().__init__(remote=remote, local=local)
         self.__sock = sock
+        self._refresh_flags(sock=sock)
+
+    # Override
+    def _get_socket(self) -> Optional[socket.socket]:
+        return self.__sock
 
     # Override
     def _set_socket(self, sock: Optional[socket.socket]):
-        super()._set_socket(sock=sock)
+        # 1. check old socket
+        old = self._get_socket()
+        if old is not None and old is not sock:
+            if is_opened(sock=old):
+                close_socket(sock=old)
+        # 2. set new socket
         self.__sock = sock
 
     # Override
-    def _close_socket(self, sock: socket.socket):
-        # DON'T close bound socket
-        if is_opened(sock=sock) and is_connected(sock=sock):
-            # sock.shutdown(socket.SHUT_RDWR)
-            sock.close()
+    def disconnect(self) -> Optional[socket.socket]:
+        # DON'T close bound socket channel
+        if self.bound and not self.connected:
+            return None
+        super().disconnect()
 
     # Override
     def close(self):
         # DON'T close bound socket channel
-        if self.connected:
-            super().close()
+        if self.bound and not self.connected:
+            return False
+        super().close()
 
     # Override
     def _create_reader(self):
@@ -129,3 +141,14 @@ class PackageChannel(BaseChannel):
     # Override
     def _create_writer(self):
         return PackageChannelWriter(channel=self)
+
+
+def close_socket(sock: socket.socket):
+    # DON'T close bound socket
+    if is_bound(sock=sock) and not is_connected(sock=sock):
+        return False
+    try:
+        # sock.shutdown(socket.SHUT_RDWR)
+        sock.close()
+    except socket.error:
+        pass
